/* Quadratic sRGB Resampler v0.11
 * By DOLLS
 * Do not distribute a modified version without permission. 
 */

texture tex1, tex2;
float2 rcpres;
sampler s0 = sampler_state {
	texture = <tex1>;
	AddressU = mirror;
	AddressV = mirror;
};

sampler s1 = sampler_state {
	texture = <tex2>;
	AddressU = mirror;
	AddressV = mirror;
};

// Linear RGB to sRGB and viceversa.
#define	GAMMA		2.35
#define	INV_GAMMA	1.0 / GAMMA

float4 Linear(float4 color) {
	return float4((color.r <= 0.0031308)? 12.92 * color.r : pow(1.055 * color.r, INV_GAMMA) - 0.055, (color.g <= 0.0031308)? 12.92 * color.g : pow(1.055 * color.g, INV_GAMMA) - 0.055, (color.b <= 0.0031308)? 12.92 * color.b : pow(1.055 * color.b, INV_GAMMA) - 0.055, color.a);
}

float4 Nonlinear(float4 color) {
	return float4((color.r <= 0.04045)? color.r / 12.92 : pow((color.r + 0.055) / 1.055, GAMMA), (color.g <= 0.04045)? color.r / 12.92 : pow((color.g + 0.055) / 1.055, GAMMA), (color.b <= 0.04045)? color.b / 12.92 : pow((color.b + 0.055) / 1.055, GAMMA), color.a);	
}

int2 res;

float4 DrawCheckerboard(in float2 coord : TEXCOORD0): COLOR0
{
	int2 pix = floor(coord / rcpres);
	float lol = (pix.x + pix.y) % 2;
	return float4(1.0 - lol, lol, 1.0 - lol, 1.0);
}

float4 Normalize(in float2 coord : TEXCOORD0): COLOR0
{		
	res = int2(256, (frac(1 / (224 * rcpres.y)) < 0.05)? 224 : 239);
	float2 loc = coord * float2(rcpres.y * res.y / (rcpres.x * res.x), 1.0) / (res.y * rcpres.y);
	return Nonlinear(tex2D(s0, loc));
}

float pow2(float x)
{
   return x * x;
}

float getWeight(float x)
{	
	if (x == 0.0) return 1.0;
	else if (x < 0.5) return 1 - 2 * pow2(x);
	else if (x < 1.0) return 2 * pow2(1 - x);
	else return 0.0;	
}

float4 Resample(in float2 coord : TEXCOORD0) : COLOR0
{	
	res = int2(256, (frac(1 / (224 * rcpres.y)) < 0.05)? 224 : 239);
	float2 pos = coord * res - 0.5;
	float2 tex = (floor(pos) + 0.5) * rcpres;
	float2 next_tex = tex + rcpres;
		
	float2 delta = frac(pos);
	float2 w0 = float2(getWeight(delta.x), getWeight(delta.y));
	float2 w1 = float2(getWeight(1 - delta.x), getWeight(1 - delta.y));
	
	float4 value = w0.x * w0.y * tex2D(s1, tex);
	value += w1.x * w0.y * tex2D(s1, float2(next_tex.x, tex.y));
	value += w0.x * w1.y * tex2D(s1, float2(tex.x, next_tex.y));
	value += w1.x * w1.y * tex2D(s1, next_tex);
	return Linear(value);
}

Technique T0
{		
	pass p0 {		
		PixelShader = compile ps_2_0 Normalize();
	}		
	pass p1 {
		PixelShader = compile ps_2_a Resample();
	}		
} 